package eu.dnetlib.ariadneplus.reader;

import java.io.StringWriter;

import org.eclipse.rdf4j.model.IRI;
import org.eclipse.rdf4j.model.Literal;
import org.eclipse.rdf4j.model.Model;
import org.eclipse.rdf4j.model.Resource;
import org.eclipse.rdf4j.model.ValueFactory;
import org.eclipse.rdf4j.model.impl.SimpleValueFactory;
import org.eclipse.rdf4j.query.GraphQuery;
import org.eclipse.rdf4j.query.GraphQueryResult;
import org.eclipse.rdf4j.query.QueryLanguage;
import org.eclipse.rdf4j.query.QueryResults;
import org.eclipse.rdf4j.repository.Repository;
import org.eclipse.rdf4j.repository.RepositoryConnection;
import org.eclipse.rdf4j.repository.manager.RemoteRepositoryManager;
import org.eclipse.rdf4j.rio.RDFFormat;
import org.eclipse.rdf4j.rio.RDFWriter;
import org.eclipse.rdf4j.rio.Rio;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import eu.dnetlib.ariadneplus.elasticsearch.BulkUpload;
import eu.dnetlib.ariadneplus.reader.json.ParseRDFJSON;

@Service
public class RunSPARQLQueryService {

    @Value("${sparql.query3:undefined}")
    private String query3;

    @Value("${repository.url:undefined}")
    private String repository_url;

    private RepositoryConnection connection;
    private RemoteRepositoryManager manager;
    private Repository repository;

    private ParseRDFJSON parser;
    private ResourceManager resourceManager;
    private BulkUpload bulkUpload;

    private void openConnection(){
    	manager = new RemoteRepositoryManager(repository_url);
		manager.init();
		manager.setUsernameAndPassword("writer", "Writer01");
//		repository = manager.getRepository("test01");
		repository = manager.getRepository("ariadneplus-ts01");
        connection =  repository.getConnection();
        
    }

    private void closeConnection(){
        connection.close();
        repository.shutDown();
        manager.shutDown();
    }

    public String executeQueryGraph(){
        if (query3.equals("undefined"))
            return null;
        openConnection();
        StringWriter recordWriter = null;
        Model model = null;
        String jsonRecord = null;
        try {
            GraphQuery graphQuery = connection.prepareGraphQuery(QueryLanguage.SPARQL, query3);
            GraphQueryResult graphQueryResult = graphQuery.evaluate();
            System.out.println("as model ...");
            model = QueryResults.asModel(graphQueryResult);
            graphQueryResult.close();
            int resourceCount = 0;
            System.out.println("counting resources  ... " );
            ValueFactory factory = SimpleValueFactory.getInstance();
            IRI iri = factory.createIRI("http://www.myprefix/resourcetype");
            Literal value = factory.createLiteral("AriadneCatalogEntry");
            for (Resource record: model.filter(null, iri, value).subjects()) {
            	System.out.println();
            	Model recordModel = null;
//            	RDFWriter rdfRecordWriter = null;
            	resourceCount+=1;
//            	if (resourceCount==12) {
//	        		break;
//	        	}
	        	recordModel = model.filter(record, null, null);
	        	if (recordModel!=null && !recordModel.isEmpty()) {
		        	recordWriter = new StringWriter();
		        	RDFWriter rdfRecordWriter = Rio.createWriter(RDFFormat.RDFJSON, recordWriter);
		        	Rio.write(recordModel, rdfRecordWriter);
//		        	System.out.println("record json: "+ recordWriter.toString());
		        	parser.parse(recordWriter.toString());
		            resourceManager.manage(parser);
		            bulkUpload.index(resourceManager);
		        }
        	}
            System.out.println();
            System.out.println("count resources: "+ resourceCount);
            System.out.println("count statements: " + model.size());
            System.out.println("index to Elastic Search completed");
        }catch(Exception e){
            e.printStackTrace();
        }finally{
            closeConnection();
            if (model!=null) {
            	model.clear();
            }
        }
        return jsonRecord;
    }

	
	public ParseRDFJSON getParser() {
		return parser;
	}

	
	public void setParser(ParseRDFJSON parser) {
		this.parser = parser;
	}

	
	public ResourceManager getResourceManager() {
		return resourceManager;
	}

	
	public void setResourceManager(ResourceManager resourceManager) {
		this.resourceManager = resourceManager;
	}

	
	public BulkUpload getBulkUpload() {
		return bulkUpload;
	}

	
	public void setBulkUpload(BulkUpload bulkUpload) {
		this.bulkUpload = bulkUpload;
	}

}
